@@ -70,7 +70,7 @@ static int set_route(struct connman_element *element, const char *gateway)
 {
 	struct ifreq ifr;
 	struct rtentry rt;
-	struct sockaddr_in *addr;
+	struct sockaddr_in addr;
 	int sk, err;
 
 	DBG("element %p", element);
@@ -92,17 +92,20 @@ static int set_route(struct connman_element *element, const char *gateway)
 	memset(&rt, 0, sizeof(rt));
 	rt.rt_flags = RTF_UP | RTF_HOST;
 
-	addr = (struct sockaddr_in *) &rt.rt_dst;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = inet_addr(gateway);
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = inet_addr(gateway);
+	memcpy(&rt.rt_dst, &addr, sizeof(rt.rt_dst));
 
-	addr = (struct sockaddr_in *) &rt.rt_gateway;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&rt.rt_gateway, &addr, sizeof(rt.rt_gateway));
 
-	addr = (struct sockaddr_in *) &rt.rt_genmask;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&rt.rt_genmask, &addr, sizeof(rt.rt_genmask));
 
 	rt.rt_dev = ifr.ifr_name;
 
@@ -114,17 +117,20 @@ static int set_route(struct connman_element *element, const char *gateway)
 	memset(&rt, 0, sizeof(rt));
 	rt.rt_flags = RTF_UP | RTF_GATEWAY;
 
-	addr = (struct sockaddr_in *) &rt.rt_dst;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&rt.rt_dst, &addr, sizeof(rt.rt_dst));
 
-	addr = (struct sockaddr_in *) &rt.rt_gateway;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = inet_addr(gateway);
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = inet_addr(gateway);
+	memcpy(&rt.rt_gateway, &addr, sizeof(rt.rt_gateway));
 
-	addr = (struct sockaddr_in *) &rt.rt_genmask;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&rt.rt_genmask, &addr, sizeof(rt.rt_genmask));
 
 	err = ioctl(sk, SIOCADDRT, &rt);
 	if (err < 0)
@@ -140,7 +146,7 @@ static int del_route(struct connman_element *element, const char *gateway)
 {
 	struct ifreq ifr;
 	struct rtentry rt;
-	struct sockaddr_in *addr;
+	struct sockaddr_in addr;
 	int sk, err;
 
 	DBG("element %p", element);
@@ -162,17 +168,20 @@ static int del_route(struct connman_element *element, const char *gateway)
 	memset(&rt, 0, sizeof(rt));
 	rt.rt_flags = RTF_UP | RTF_GATEWAY;
 
-	addr = (struct sockaddr_in *) &rt.rt_dst;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&rt.rt_dst, &addr, sizeof(rt.rt_dst));
 
-	addr = (struct sockaddr_in *) &rt.rt_gateway;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = inet_addr(gateway);
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = inet_addr(gateway);
+	memcpy(&rt.rt_gateway, &addr, sizeof(rt.rt_gateway));
 
-	addr = (struct sockaddr_in *) &rt.rt_genmask;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&rt.rt_genmask, &addr, sizeof(rt.rt_genmask));
 
 	err = ioctl(sk, SIOCDELRT, &rt);
 	if (err < 0)
@@ -243,8 +252,7 @@ static struct gateway_data *add_gateway(int index, const char *gateway)
 							find_element, data);
 
 	service = __connman_element_get_service(data->element);
-	if (service != NULL)
-		data->order = __connman_service_get_order(service);
+	data->order = __connman_service_get_order(service);
 
 	gateway_list = g_slist_append(gateway_list, data);
 
@@ -281,12 +289,16 @@ static void set_default_gateway(struct gateway_data *data)
 static struct gateway_data *find_default_gateway(void)
 {
 	struct gateway_data *found = NULL;
+	unsigned int order = 0;
 	GSList *list;
 
 	for (list = gateway_list; list; list = list->next) {
 		struct gateway_data *data = list->data;
-		/* just return the last one for now */
-		found = data;
+
+		if (found == NULL || data->order > order) {
+			found = data;
+			order = data->order;
+		}
 	}
 
 	return found;
@@ -526,9 +538,11 @@ static int connection_probe(struct connman_element *element)
 
 	if (register_interface(element) < 0)
 		return -ENODEV;
+
 	service = __connman_element_get_service(element);
 	__connman_service_indicate_state(service,
 					CONNMAN_SERVICE_STATE_READY);
+
 	connman_element_set_enabled(element, TRUE);
 	emit_default_signal(element);
 
@@ -640,20 +654,24 @@ static void update_order(void)
 		struct connman_service *service;
 
 		service = __connman_element_get_service(data->element);
-
 		data->order = __connman_service_get_order(service);
 	}
 }
 
-void __connman_connection_update_gateway(void)
+gboolean __connman_connection_update_gateway(void)
 {
 	struct gateway_data *active_gateway, *default_gateway;
+	gboolean updated = FALSE;
 
 	update_order();
 
 	active_gateway = find_active_gateway();
 	default_gateway = find_default_gateway();
 
-	if (active_gateway && active_gateway != default_gateway)
+	if (active_gateway && active_gateway != default_gateway) {
 		del_route(active_gateway->element, active_gateway->gateway);
+		updated = TRUE;
+	}
+
+	return updated;
 }
