@@ -34,10 +34,10 @@ struct connman_network {
 	struct connman_element element;
 	enum connman_network_type type;
 	enum connman_network_protocol protocol;
-	connman_bool_t associating;
 	connman_bool_t secondary;
 	connman_bool_t available;
 	connman_bool_t connected;
+	connman_bool_t hidden;
 	connman_uint8_t strength;
 	connman_uint16_t frequency;
 	char *identifier;
@@ -51,6 +51,8 @@ struct connman_network {
 	void *driver_data;
 
 	connman_bool_t registered;
+	connman_bool_t connecting;
+	connman_bool_t associating;
 
 	struct connman_device *device;
 
@@ -77,6 +79,7 @@ static const char *type2string(enum connman_network_type type)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 		return "bluetooth";
+	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 		return "cellular";
 	}
@@ -209,76 +212,12 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else
 		return __connman_error_invalid_property(msg);
 
-	__connman_storage_save_network(network);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *do_connect(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_network *network = data;
-	int err;
-
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (network->connected == TRUE)
-		return __connman_error_failed(msg, EALREADY);
-
-	if (network->driver && network->driver->connect) {
-		enum connman_device_mode mode;
-
-		mode = connman_device_get_mode(network->device);
-		if (mode == CONNMAN_DEVICE_MODE_NETWORK_SINGLE)
-			__connman_device_disconnect(network->device);
-
-		err = network->driver->connect(network);
-		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg, -err);
-	} else
-		network->connected = TRUE;
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *do_disconnect(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_network *network = data;
-	int err;
-
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (network->connected == FALSE)
-		return __connman_error_failed(msg, EINVAL);
-
-	connman_element_unregister_children(&network->element);
-
-	connman_device_set_disconnected(network->device, TRUE);
-
-	if (network->driver && network->driver->disconnect) {
-		err = network->driver->disconnect(network);
-		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg, -err);
-	} else
-		network->connected = FALSE;
-
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
 static GDBusMethodTable network_methods[] = {
 	{ "GetProperties", "",   "a{sv}", get_properties },
 	{ "SetProperty",   "sv", "",      set_property   },
-	{ "Connect",       "",   "",      do_connect     },
-	{ "Disconnect",    "",   "",      do_disconnect  },
 	{ },
 };
 
@@ -459,7 +398,12 @@ struct connman_network *connman_network_create(const char *identifier,
 	__connman_element_initialize(&network->element);
 
 	//temp = connman_dbus_encode_string(identifier);
-	temp = g_strdup(identifier);
+	if (identifier == NULL) {
+		temp = g_strdup("hidden");
+		network->hidden = TRUE;
+	} else
+		temp = g_strdup(identifier);
+
 	if (temp == NULL) {
 		g_free(network);
 		return NULL;
@@ -479,7 +423,7 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	network->type       = type;
 	network->secondary  = FALSE;
-	network->identifier = g_strdup(identifier);
+	network->identifier = g_strdup(temp);
 
 	network->ipconfig = connman_ipconfig_create();
 	if (network->ipconfig == NULL) {
@@ -602,21 +546,25 @@ void connman_network_set_group(struct connman_network *network,
 	if (network->secondary == TRUE)
 		return;
 
-	if (g_strcmp0(network->group, group) == 0)
-		return;
-
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-	case CONNMAN_NETWORK_TYPE_HSO:
 		return;
+	case CONNMAN_NETWORK_TYPE_MBM:
+	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		break;
 	}
 
+	if (g_strcmp0(network->group, group) == 0) {
+		if (group != NULL)
+			__connman_profile_update_network(network);
+		return;
+	}
+
 	if (network->group != NULL) {
 		__connman_profile_remove_network(network);
 
@@ -629,7 +577,13 @@ void connman_network_set_group(struct connman_network *network,
 		__connman_profile_add_network(network);
 }
 
-const char *__connman_network_get_group(struct connman_network *network)
+/**
+ * connman_network_get_group:
+ * @network: network structure
+ *
+ * Get group name for automatic clustering
+ */
+const char *connman_network_get_group(struct connman_network *network)
 {
 	return network->group;
 }
@@ -642,6 +596,29 @@ const char *__connman_network_get_ident(struct connman_network *network)
 	return __connman_device_get_ident(network->device);
 }
 
+connman_bool_t __connman_network_get_weakness(struct connman_network *network)
+{
+	if (network->secondary == TRUE)
+		return FALSE;
+
+	switch (network->type) {
+	case CONNMAN_NETWORK_TYPE_UNKNOWN:
+	case CONNMAN_NETWORK_TYPE_VENDOR:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_MBM:
+	case CONNMAN_NETWORK_TYPE_HSO:
+	case CONNMAN_NETWORK_TYPE_WIMAX:
+		break;
+	case CONNMAN_NETWORK_TYPE_WIFI:
+		if (network->strength > 0 && network->strength < 20)
+			return TRUE;
+		break;
+	}
+
+	return FALSE;
+}
+
 /**
  * connman_network_set_available:
  * @network: network structure
@@ -670,6 +647,9 @@ int connman_network_set_available(struct connman_network *network,
  */
 connman_bool_t connman_network_get_available(struct connman_network *network)
 {
+	if (network->hidden == TRUE)
+		return TRUE;
+
 	return network->available;
 }
 
@@ -745,6 +725,7 @@ static gboolean set_connected(gpointer user_data)
 		connman_element_unregister_children(&network->element);
 
 		__connman_device_set_network(network->device, NULL);
+		network->hidden = FALSE;
 
 		__connman_device_decrease_connections(network->device);
 
@@ -752,6 +733,10 @@ static gboolean set_connected(gpointer user_data)
 						CONNMAN_SERVICE_STATE_IDLE);
 	}
 
+	network->connecting = FALSE;
+
+	connman_network_set_associating(network, FALSE);
+
 	return FALSE;
 }
 
@@ -771,14 +756,18 @@ int connman_network_set_connected(struct connman_network *network,
 
 	DBG("network %p connected %d", network, connected);
 
+	if ((network->connecting == TRUE || network->associating == TRUE) &&
+							connected == FALSE) {
+		connman_element_set_error(&network->element,
+					CONNMAN_ELEMENT_ERROR_CONNECT_FAILED);
+		__connman_network_disconnect(network);
+	}
+
 	if (network->connected == connected)
 		return -EALREADY;
 
 	network->connected = connected;
 
-	if (connected == TRUE)
-		network->associating = FALSE;
-
 	if (network->registered == FALSE) {
 		g_idle_add(set_connected, network);
 		return 0;
@@ -816,6 +805,17 @@ connman_bool_t connman_network_get_connected(struct connman_network *network)
 	return network->connected;
 }
 
+/**
+ * connman_network_get_associating:
+ * @network: network structure
+ *
+ * Get network associating status
+ */
+connman_bool_t connman_network_get_associating(struct connman_network *network)
+{
+	return network->associating;
+}
+
 /**
  * __connman_network_connect:
  * @network: network structure
@@ -829,6 +829,9 @@ int __connman_network_connect(struct connman_network *network)
 	DBG("network %p", network);
 
 	if (network->connected == TRUE)
+		return -EISCONN;
+
+	if (network->connecting == TRUE || network->associating == TRUE)
 		return -EALREADY;
 
 	if (network->driver == NULL)
@@ -839,12 +842,21 @@ int __connman_network_connect(struct connman_network *network)
 
 	__connman_device_disconnect(network->device);
 
+	network->connecting = TRUE;
+
 	err = network->driver->connect(network);
-	if (err == 0) {
-		network->connected = TRUE;
-		set_connected(network);
+	if (err < 0) {
+		if (err == -EINPROGRESS)
+			connman_network_set_associating(network, TRUE);
+		else
+			network->hidden = FALSE;
+
+		return err;
 	}
 
+	network->connected = TRUE;
+	set_connected(network);
+
 	return err;
 }
 
@@ -860,7 +872,8 @@ int __connman_network_disconnect(struct connman_network *network)
 
 	DBG("network %p", network);
 
-	if (network->connected == FALSE)
+	if (network->connected == FALSE && network->connecting == FALSE &&
+						network->associating == FALSE)
 		return -ENOTCONN;
 
 	if (network->driver == NULL)
@@ -869,6 +882,8 @@ int __connman_network_disconnect(struct connman_network *network)
 	if (network->driver->disconnect == NULL)
 		return -ENOSYS;
 
+	network->connecting = FALSE;
+
 	err = network->driver->disconnect(network);
 	if (err == 0) {
 		network->connected = FALSE;
@@ -1232,8 +1247,9 @@ static int network_probe(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-	case CONNMAN_NETWORK_TYPE_HSO:
 		break;
+	case CONNMAN_NETWORK_TYPE_MBM:
+	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		if (network->group != NULL && network->secondary == FALSE)
@@ -1261,8 +1277,9 @@ static void network_remove(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-	case CONNMAN_NETWORK_TYPE_HSO:
 		break;
+	case CONNMAN_NETWORK_TYPE_MBM:
+	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		if (network->group != NULL && network->secondary == FALSE) {
