@@ -64,7 +64,7 @@ static void append_path(gpointer value, gpointer user_data)
 	struct connman_service *service = value;
 	DBusMessageIter *iter = user_data;
 
-	if (service->path == NULL)
+	if (service->path == NULL || service->hidden == TRUE)
 		return;
 
 	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
@@ -187,6 +187,8 @@ static const char *error2string(enum connman_service_error error)
 	switch (error) {
 	case CONNMAN_SERVICE_ERROR_UNKNOWN:
 		break;
+	case CONNMAN_SERVICE_ERROR_OUT_OF_RANGE:
+		return "out-of-range";
 	case CONNMAN_SERVICE_ERROR_PIN_MISSING:
 		return "pin-missing";
 	case CONNMAN_SERVICE_ERROR_DHCP_FAILED:
@@ -208,6 +210,26 @@ static enum connman_service_error string2error(const char *error)
 	return CONNMAN_SERVICE_ERROR_UNKNOWN;
 }
 
+const char *__connman_service_default(void)
+{
+	struct connman_service *service;
+	GSequenceIter *iter;
+
+	iter = g_sequence_get_begin_iter(service_list);
+
+	if (g_sequence_iter_is_end(iter) == TRUE)
+		return "";
+
+	service = g_sequence_get(iter);
+	if (service == NULL)
+		return "";
+
+	if (service->state != CONNMAN_SERVICE_STATE_READY)
+		return "";
+
+	return type2string(service->type);
+}
+
 static void state_changed(struct connman_service *service)
 {
 	DBusMessage *signal;
@@ -268,12 +290,64 @@ static void strength_changed(struct connman_service *service)
 	g_dbus_send_message(connection, signal);
 }
 
+static void passphrase_changed(struct connman_service *service)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	dbus_bool_t required;
+	const char *key = "PassphraseRequired";
+
+	if (service->path == NULL)
+		return;
+
+	switch (service->type) {
+	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_WIMAX:
+	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_CELLULAR:
+		return;
+	case CONNMAN_SERVICE_TYPE_WIFI:
+		required = FALSE;
+
+		switch (service->security) {
+		case CONNMAN_SERVICE_SECURITY_UNKNOWN:
+		case CONNMAN_SERVICE_SECURITY_NONE:
+			break;
+		case CONNMAN_SERVICE_SECURITY_WEP:
+		case CONNMAN_SERVICE_SECURITY_WPA:
+		case CONNMAN_SERVICE_SECURITY_RSN:
+			if (service->passphrase == NULL)
+				required = TRUE;
+			break;
+		}
+		break;
+	}
+
+	signal = dbus_message_new_signal(service->path,
+				CONNMAN_SERVICE_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &required);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+}
+
 static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_service *service = user_data;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
+	dbus_bool_t required;
 	const char *str;
 
 	DBG("service %p", service);
@@ -325,12 +399,39 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_variant(&dict, "Name",
 					DBUS_TYPE_STRING, &service->name);
 
-	if (service->passphrase != NULL &&
-			__connman_security_check_privilege(msg,
-				CONNMAN_SECURITY_PRIVILEGE_SECRET) == 0)
-		connman_dbus_dict_append_variant(&dict, "Passphrase",
+	switch (service->type) {
+	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_WIMAX:
+	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_CELLULAR:
+		break;
+	case CONNMAN_SERVICE_TYPE_WIFI:
+		if (service->passphrase != NULL &&
+				__connman_security_check_privilege(msg,
+					CONNMAN_SECURITY_PRIVILEGE_SECRET) == 0)
+			connman_dbus_dict_append_variant(&dict, "Passphrase",
 				DBUS_TYPE_STRING, &service->passphrase);
 
+		required = FALSE;
+
+		switch (service->security) {
+		case CONNMAN_SERVICE_SECURITY_UNKNOWN:
+		case CONNMAN_SERVICE_SECURITY_NONE:
+			break;
+		case CONNMAN_SERVICE_SECURITY_WEP:
+		case CONNMAN_SERVICE_SECURITY_WPA:
+		case CONNMAN_SERVICE_SECURITY_RSN:
+			if (service->passphrase == NULL)
+				required = TRUE;
+			break;
+		}
+
+		connman_dbus_dict_append_variant(&dict, "PassphraseRequired",
+						DBUS_TYPE_BOOLEAN, &required);
+		break;
+	}
+
 	__connman_ipconfig_append_ipv4(service->ipconfig, &dict, "IPv4.");
 
 	dbus_message_iter_close_container(&array, &dict);
@@ -376,6 +477,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 		g_free(service->passphrase);
 		service->passphrase = g_strdup(passphrase);
 
+		passphrase_changed(service);
+
 		if (service->network != NULL)
 			connman_network_set_string(service->network,
 				"WiFi.Passphrase", service->passphrase);
@@ -394,6 +497,13 @@ static DBusMessage *set_property(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
+static void set_idle(struct connman_service *service)
+{
+	service->state = CONNMAN_SERVICE_STATE_IDLE;
+	service->error = CONNMAN_SERVICE_ERROR_UNKNOWN;
+	state_changed(service);
+}
+
 static DBusMessage *clear_property(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
@@ -410,11 +520,16 @@ static DBusMessage *clear_property(DBusConnection *conn,
 		return __connman_error_permission_denied(msg);
 
 	if (g_str_equal(name, "Error") == TRUE) {
-		service->state = CONNMAN_SERVICE_STATE_IDLE;
-		service->error = CONNMAN_SERVICE_ERROR_UNKNOWN;
-		state_changed(service);
+		set_idle(service);
 
 		g_get_current_time(&service->modified);
+		__connman_storage_save_service(service);
+	} else if (g_str_equal(name, "Passphrase") == TRUE) {
+		g_free(service->passphrase);
+		service->passphrase = NULL;
+
+		passphrase_changed(service);
+
 		__connman_storage_save_service(service);
 	} else
 		return __connman_error_invalid_property(msg);
@@ -440,24 +555,6 @@ static connman_bool_t is_connecting(struct connman_service *service)
 	return FALSE;
 }
 
-static struct connman_service *find_pending_service(void)
-{
-	struct connman_service *service;
-	GSequenceIter *iter;
-
-	iter = g_sequence_get_begin_iter(service_list);
-
-	while (g_sequence_iter_is_end(iter) == FALSE) {
-		service = g_sequence_get(iter);
-		if (service->pending != NULL)
-			return service;
-
-		iter = g_sequence_iter_next(iter);
-	}
-
-	return NULL;
-}
-
 static connman_bool_t is_ignore(struct connman_service *service)
 {
 	if (service->ignore == TRUE)
@@ -471,41 +568,65 @@ static connman_bool_t is_ignore(struct connman_service *service)
 
 void __connman_service_auto_connect(void)
 {
-	struct connman_service *service;
+	struct connman_service *service = NULL;
 	GSequenceIter *iter;
 
 	DBG("");
 
-	service = find_pending_service();
-	if (service != NULL)
-		return;
-
 	iter = g_sequence_get_begin_iter(service_list);
-	if (g_sequence_iter_is_end(iter) == TRUE)
-		return;
 
-	service = g_sequence_get(iter);
-
-	while (is_ignore(service) == TRUE) {
-		iter = g_sequence_iter_next(iter);
-		if (g_sequence_iter_is_end(iter))
-			return;
+	while (g_sequence_iter_is_end(iter) == FALSE) {
 		service = g_sequence_get(iter);
+
+		if (service->pending != NULL)
+			return;
+
+		if (is_connecting(service) == TRUE)
+			return;
+
+		if (service->favorite == FALSE)
+			return;
+
+		if (service->state == CONNMAN_SERVICE_STATE_READY)
+			return;
+
+		if (is_ignore(service) == FALSE &&
+				service->state == CONNMAN_SERVICE_STATE_IDLE)
+			break;
+
+		service = NULL;
+
+		iter = g_sequence_iter_next(iter);
 	}
 
-	if (service->favorite == FALSE)
-		return;
-
-	if (service->state == CONNMAN_SERVICE_STATE_READY)
-		return;
-
-	if (is_connecting(service) == TRUE)
-		return;
-
-	if (service->state == CONNMAN_SERVICE_STATE_IDLE)
+	if (service != NULL)
 		__connman_service_connect(service);
 }
 
+static void reply_pending(struct connman_service *service, int error)
+{
+	if (service->timeout > 0) {
+		g_source_remove(service->timeout);
+		service->timeout = 0;
+	}
+
+	if (service->pending != NULL) {
+		if (error > 0) {
+			DBusMessage *reply;
+
+			reply = __connman_error_failed(service->pending,
+								error);
+			if (reply != NULL)
+				g_dbus_send_message(connection, reply);
+		} else
+			g_dbus_send_reply(connection, service->pending,
+							DBUS_TYPE_INVALID);
+
+		dbus_message_unref(service->pending);
+		service->pending = NULL;
+	}
+}
+
 static gboolean connect_timeout(gpointer user_data)
 {
 	struct connman_service *service = user_data;
@@ -564,6 +685,7 @@ static DBusMessage *connect_service(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_service *service = user_data;
+	GSequenceIter *iter;
 	int err;
 
 	DBG("service %p", service);
@@ -571,6 +693,18 @@ static DBusMessage *connect_service(DBusConnection *conn,
 	if (service->pending != NULL)
 		return __connman_error_in_progress(msg);
 
+	iter = g_sequence_get_begin_iter(service_list);
+
+	while (g_sequence_iter_is_end(iter) == FALSE) {
+		struct connman_service *temp = g_sequence_get(iter);
+
+		if (service->type == temp->type &&
+					is_connecting(temp) == TRUE)
+			return __connman_error_in_progress(msg);
+
+		iter = g_sequence_iter_next(iter);
+	}
+
 	service->ignore = FALSE;
 
 	service->pending = dbus_message_ref(msg);
@@ -601,19 +735,6 @@ static DBusMessage *disconnect_service(DBusConnection *conn,
 
 	DBG("service %p", service);
 
-	if (service->pending != NULL) {
-		DBusMessage *reply;
-
-		reply = __connman_error_operation_aborted(service->pending);
-		if (reply != NULL)
-			g_dbus_send_message(conn, reply);
-
-		dbus_message_unref(service->pending);
-		service->pending = NULL;
-
-		return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-	}
-
 	service->ignore = TRUE;
 
 	err = __connman_service_disconnect(service);
@@ -646,6 +767,8 @@ static DBusMessage *remove_service(DBusConnection *conn,
 	g_free(service->passphrase);
 	service->passphrase = NULL;
 
+	passphrase_changed(service);
+
 	connman_service_set_favorite(service, FALSE);
 	__connman_storage_save_service(service);
 
@@ -739,22 +862,14 @@ static void service_free(gpointer user_data)
 
 	DBG("service %p", service);
 
+	reply_pending(service, ENOENT);
+
 	g_hash_table_remove(service_hash, service->identifier);
 
-	if (service->timeout > 0) {
-		g_source_remove(service->timeout);
-		service->timeout = 0;
-	}
-
-	if (service->pending != NULL) {
-		dbus_message_unref(service->pending);
-		service->pending = NULL;
-	}
-
 	service->path = NULL;
 
 	if (path != NULL) {
-		__connman_profile_changed(TRUE);
+		__connman_profile_changed(FALSE);
 
 		g_dbus_unregister_interface(connection, path,
 						CONNMAN_SERVICE_INTERFACE);
@@ -787,9 +902,10 @@ void __connman_service_put(struct connman_service *service)
 		GSequenceIter *iter;
 
 		iter = g_hash_table_lookup(service_hash, service->identifier);
-		if (iter != NULL)
+		if (iter != NULL) {
+			__connman_service_disconnect(service);
 			g_sequence_remove(iter);
-		else
+		} else
 			service_free(service);
 	}
 }
@@ -924,37 +1040,28 @@ int connman_service_set_favorite(struct connman_service *service,
 	return 0;
 }
 
-int __connman_service_set_carrier(struct connman_service *service,
-						connman_bool_t carrier)
+static void default_changed(void)
 {
-	DBG("service %p carrier %d", service, carrier);
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "DefaultTechnology";
+	const char *str = __connman_service_default();
 
-	if (service == NULL)
-		return -EINVAL;
+	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
 
-	switch (service->type) {
-	case CONNMAN_SERVICE_TYPE_UNKNOWN:
-	case CONNMAN_SERVICE_TYPE_WIFI:
-	case CONNMAN_SERVICE_TYPE_WIMAX:
-	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
-	case CONNMAN_SERVICE_TYPE_CELLULAR:
-		return -EINVAL;
-	case CONNMAN_SERVICE_TYPE_ETHERNET:
-		break;
-	}
+	dbus_message_iter_init_append(signal, &entry);
 
-	if (carrier == FALSE) {
-		service->state = CONNMAN_SERVICE_STATE_DISCONNECT;
-		state_changed(service);
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
 
-		service->state = CONNMAN_SERVICE_STATE_IDLE;
-		state_changed(service);
-	} else {
-		service->state = CONNMAN_SERVICE_STATE_CARRIER;
-		state_changed(service);
-	}
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_STRING_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_STRING, &str);
+	dbus_message_iter_close_container(&entry, &value);
 
-	return connman_service_set_favorite(service, carrier);
+	g_dbus_send_message(connection, signal);
 }
 
 int __connman_service_indicate_state(struct connman_service *service,
@@ -967,9 +1074,6 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (service == NULL)
 		return -EINVAL;
 
-	if (state == CONNMAN_SERVICE_STATE_CARRIER)
-		return __connman_service_set_carrier(service, TRUE);
-
 	if (service->state == state)
 		return -EALREADY;
 
@@ -985,6 +1089,8 @@ int __connman_service_indicate_state(struct connman_service *service,
 			service->state != CONNMAN_SERVICE_STATE_DISCONNECT) {
 		service->state = CONNMAN_SERVICE_STATE_DISCONNECT;
 		state_changed(service);
+
+		__connman_service_disconnect(service);
 	}
 
 	service->state = state;
@@ -993,39 +1099,22 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (state == CONNMAN_SERVICE_STATE_READY) {
 		connman_service_set_favorite(service, TRUE);
 
-		if (service->timeout > 0) {
-			g_source_remove(service->timeout);
-			service->timeout = 0;
-		}
-
-		if (service->pending != NULL) {
-			g_dbus_send_reply(connection, service->pending,
-							DBUS_TYPE_INVALID);
-
-			dbus_message_unref(service->pending);
-			service->pending = NULL;
-		}
+		reply_pending(service, 0);
 
 		g_get_current_time(&service->modified);
 		__connman_storage_save_service(service);
+
+		__connman_notifier_connect(service->type);
+
+		default_changed();
+	} else if (state == CONNMAN_SERVICE_STATE_DISCONNECT) {
+		default_changed();
+
+		__connman_notifier_disconnect(service->type);
 	}
 
 	if (state == CONNMAN_SERVICE_STATE_FAILURE) {
-		if (service->timeout > 0) {
-			g_source_remove(service->timeout);
-			service->timeout = 0;
-		}
-
-		if (service->pending != NULL) {
-			DBusMessage *reply;
-
-			reply = __connman_error_failed(service->pending, EIO);
-			if (reply != NULL)
-				g_dbus_send_message(connection, reply);
-
-			dbus_message_unref(service->pending);
-			service->pending = NULL;
-		}
+		reply_pending(service, EIO);
 
 		g_get_current_time(&service->modified);
 		__connman_storage_save_service(service);
@@ -1038,6 +1127,10 @@ int __connman_service_indicate_state(struct connman_service *service,
 
 	__connman_profile_changed(FALSE);
 
+	if (service->state == CONNMAN_SERVICE_STATE_IDLE ||
+			service->state == CONNMAN_SERVICE_STATE_FAILURE)
+		__connman_element_request_scan(CONNMAN_ELEMENT_TYPE_UNKNOWN);
+
 	return 0;
 }
 
@@ -1059,9 +1152,41 @@ int __connman_service_indicate_default(struct connman_service *service)
 {
 	DBG("service %p", service);
 
+	default_changed();
+
 	return 0;
 }
 
+static connman_bool_t prepare_network(struct connman_service *service)
+{
+	enum connman_network_type type;
+	unsigned int ssid_len;
+
+	type = connman_network_get_type(service->network);
+
+	switch (type) {
+	case CONNMAN_NETWORK_TYPE_UNKNOWN:
+	case CONNMAN_NETWORK_TYPE_VENDOR:
+		return FALSE;
+	case CONNMAN_NETWORK_TYPE_WIFI:
+		if (connman_network_get_blob(service->network, "WiFi.SSID",
+							&ssid_len) == NULL)
+			return FALSE;
+
+		connman_network_set_string(service->network,
+				"WiFi.Passphrase", service->passphrase);
+		break;
+	case CONNMAN_NETWORK_TYPE_WIMAX:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_MBM:
+	case CONNMAN_NETWORK_TYPE_HSO:
+		break;
+	}
+
+	return TRUE;
+}
+
 int __connman_service_connect(struct connman_service *service)
 {
 	int err;
@@ -1074,16 +1199,33 @@ int __connman_service_connect(struct connman_service *service)
 	if (is_connecting(service) == TRUE)
 		return -EALREADY;
 
+	switch (service->type) {
+	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+		return -EINVAL;
+	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_WIMAX:
+	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_CELLULAR:
+		break;
+	case CONNMAN_SERVICE_TYPE_WIFI:
+		switch (service->security) {
+		case CONNMAN_SERVICE_SECURITY_UNKNOWN:
+		case CONNMAN_SERVICE_SECURITY_NONE:
+			break;
+		case CONNMAN_SERVICE_SECURITY_WEP:
+		case CONNMAN_SERVICE_SECURITY_WPA:
+		case CONNMAN_SERVICE_SECURITY_RSN:
+			if (service->passphrase == NULL)
+				return -ENOKEY;
+			break;
+		}
+		break;
+	}
+
 	if (service->network != NULL) {
-		unsigned int ssid_len;
-
-		if (connman_network_get_blob(service->network, "WiFi.SSID",
-						     &ssid_len) == NULL)
+		if (prepare_network(service) == FALSE)
 			return -EINVAL;
 
-		connman_network_set_string(service->network,
-				"WiFi.Passphrase", service->passphrase);
-
 		err = __connman_network_connect(service->network);
 	} else if (service->device != NULL) {
 		if (service->favorite == FALSE)
@@ -1112,6 +1254,8 @@ int __connman_service_disconnect(struct connman_service *service)
 
 	DBG("service %p", service);
 
+	reply_pending(service, ECONNABORTED);
+
 	if (service->network != NULL) {
 		err = __connman_network_disconnect(service->network);
 	} else if (service->device != NULL) {
@@ -1148,15 +1292,72 @@ static struct connman_service *__connman_service_lookup(const char *identifier)
 	return NULL;
 }
 
+static struct connman_network *create_hidden_wifi(struct connman_device *device,
+		const char *ssid, const char *mode, const char *security)
+{
+	struct connman_network *network;
+	char *name;
+	int index;
+	unsigned int i, ssid_len;
+
+	ssid_len = strlen(ssid);
+	if (ssid_len < 1)
+		return NULL;
+
+	network = connman_network_create(NULL, CONNMAN_NETWORK_TYPE_WIFI);
+	if (network == NULL)
+		return NULL;
+
+	connman_network_set_blob(network, "WiFi.SSID",
+					(unsigned char *) ssid, ssid_len);
+
+	connman_network_set_string(network, "WiFi.Mode", mode);
+	connman_network_set_string(network, "WiFi.Security", security);
+
+	name = g_try_malloc0(ssid_len + 1);
+	if (name == NULL) {
+		connman_network_unref(network);
+		return NULL;
+	}
+
+	for (i = 0; i < ssid_len; i++) {
+		if (g_ascii_isprint(ssid[i]))
+			name[i] = ssid[i];
+		else
+			name[i] = ' ';
+	}
+
+	connman_network_set_name(network, name);
+
+	g_free(name);
+
+	index = connman_device_get_index(device);
+	connman_network_set_index(network, index);
+
+	connman_network_set_protocol(network, CONNMAN_NETWORK_PROTOCOL_IP);
+
+	if (connman_device_add_network(device, network) < 0) {
+		connman_network_unref(network);
+		return NULL;
+	}
+
+	connman_network_set_available(network, TRUE);
+
+	return network;
+}
+
 int __connman_service_create_and_connect(DBusMessage *msg)
 {
 	struct connman_service *service;
+	struct connman_network *network;
 	struct connman_device *device;
 	DBusMessageIter iter, array;
 	const char *mode = "managed", *security = "none";
 	const char *type = NULL, *ssid = NULL, *passphrase = NULL;
+	unsigned int ssid_len = 0;
 	const char *ident;
 	char *name, *group;
+	gboolean created = FALSE;
 	int err;
 
 	dbus_message_iter_init(msg, &iter);
@@ -1202,7 +1403,17 @@ int __connman_service_create_and_connect(DBusMessage *msg)
 	if (ssid == NULL)
 		return -EINVAL;
 
-	device = __connman_element_find_device(CONNMAN_DEVICE_TYPE_WIFI);
+	ssid_len = strlen(ssid);
+	if (ssid_len < 1)
+		return -EINVAL;
+
+	if (g_strcmp0(security, "none") != 0 &&
+			g_strcmp0(mode, "wep") != 0 &&
+				g_strcmp0(mode, "wpa") != 0 &&
+					g_strcmp0(mode, "rsn") != 0)
+		return -EINVAL;
+
+	device = __connman_element_find_device(CONNMAN_SERVICE_TYPE_WIFI);
 	if (device == NULL)
 		return -EOPNOTSUPP;
 
@@ -1211,36 +1422,64 @@ int __connman_service_create_and_connect(DBusMessage *msg)
 		return -EOPNOTSUPP;
 
 	group = connman_wifi_build_group_name((unsigned char *) ssid,
-						strlen(ssid), mode, security);
+						ssid_len, mode, security);
 	if (group == NULL)
 		return -EINVAL;
 
 	name = g_strdup_printf("%s_%s_%s", type, ident, group);
 
-	g_free(group);
+	service = __connman_service_lookup(name);
+
+	if (service != NULL)
+		goto done;
+
+	network = create_hidden_wifi(device, ssid, mode, security);
+	if (network != NULL) {
+		connman_network_set_group(network, group);
+		created = TRUE;
+	}
 
 	service = __connman_service_lookup(name);
 
+done:
 	g_free(name);
+	g_free(group);
 
-	if (service != NULL) {
-		if (passphrase != NULL) {
-			g_free(service->passphrase);
-			service->passphrase = g_strdup(passphrase);
-		}
-
-		err = __connman_service_connect(service);
-		if (err < 0 && err != -EINPROGRESS)
-			return err;
-
-		g_dbus_send_reply(connection, msg,
-					DBUS_TYPE_OBJECT_PATH, &service->path,
-							DBUS_TYPE_INVALID);
-
-		return 0;
+	if (service == NULL) {
+		err = -EOPNOTSUPP;
+		goto failed;
 	}
 
-	return -EOPNOTSUPP;
+	__connman_device_disconnect(device);
+
+	if (passphrase != NULL) {
+		g_free(service->passphrase);
+		service->passphrase = g_strdup(passphrase);
+	}
+
+	err = __connman_service_connect(service);
+	if (err < 0 && err != -EINPROGRESS)
+		goto failed;
+
+	g_dbus_send_reply(connection, msg,
+				DBUS_TYPE_OBJECT_PATH, &service->path,
+							DBUS_TYPE_INVALID);
+
+	return 0;
+
+failed:
+	if (service != NULL && created == TRUE) {
+		struct connman_network *network = service->network;
+
+		if (network != NULL) {
+			connman_network_set_available(network, FALSE);
+			__connman_device_cleanup_networks(device);
+		}
+
+		__connman_service_put(service);
+	}
+
+	return err;
 }
 
 /**
@@ -1313,7 +1552,7 @@ static int service_register(struct connman_service *service)
 }
 
 /**
- * connman_service_lookup_from_device:
+ * __connman_service_lookup_from_device:
  * @device: device structure
  *
  * Look up a service by device (reference count will not be increased)
@@ -1330,40 +1569,14 @@ struct connman_service *__connman_service_lookup_from_device(struct connman_devi
 
 	name = g_strdup_printf("%s_%s",
 				__connman_device_get_type(device), ident);
-
 	service = __connman_service_lookup(name);
-
 	g_free(name);
 
 	return service;
 }
 
-static enum connman_service_type convert_device_type(struct connman_device *device)
-{
-	enum connman_device_type type = connman_device_get_type(device);
-
-	switch (type) {
-	case CONNMAN_DEVICE_TYPE_UNKNOWN:
-	case CONNMAN_DEVICE_TYPE_VENDOR:
-	case CONNMAN_DEVICE_TYPE_WIFI:
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_MBM:
-	case CONNMAN_DEVICE_TYPE_HSO:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
-		break;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
-		return CONNMAN_SERVICE_TYPE_ETHERNET;
-	}
-
-	return CONNMAN_SERVICE_TYPE_UNKNOWN;
-}
-
 /**
- * connman_service_create_from_device:
+ * __connman_service_create_from_device:
  * @device: device structure
  *
  * Look up service by device and if not found, create one
@@ -1380,34 +1593,53 @@ struct connman_service *__connman_service_create_from_device(struct connman_devi
 
 	name = g_strdup_printf("%s_%s",
 				__connman_device_get_type(device), ident);
-
 	service = __connman_service_get(name);
+	g_free(name);
+
 	if (service == NULL)
-		goto done;
+		return NULL;
 
 	if (service->path != NULL) {
-		__connman_service_put(service);
-		service = NULL;
-		goto done;
+		__connman_profile_changed(TRUE);
+		return service;
 	}
 
-	service->type = convert_device_type(device);
+	service->type = __connman_device_get_service_type(device);
 
 	service->device = device;
 
 	service_register(service);
 
+	__connman_profile_changed(TRUE);
+
 	if (service->favorite == TRUE)
 		__connman_service_auto_connect();
 
-done:
-	g_free(name);
-
 	return service;
 }
 
+void __connman_service_remove_from_device(struct connman_device *device)
+{
+	struct connman_service *service;
+	enum connman_service_type type;
+
+	service = __connman_service_lookup_from_device(device);
+	if (service == NULL)
+		return;
+
+	type = service->type;
+
+	__connman_service_put(service);
+
+	default_changed();
+
+	__connman_notifier_disconnect(type);
+
+	__connman_service_auto_connect();
+}
+
 /**
- * connman_service_lookup_from_network:
+ * __connman_service_lookup_from_network:
  * @network: network structure
  *
  * Look up a service by network (reference count will not be increased)
@@ -1428,9 +1660,7 @@ struct connman_service *__connman_service_lookup_from_network(struct connman_net
 
 	name = g_strdup_printf("%s_%s_%s",
 			__connman_network_get_type(network), ident, group);
-
 	service = __connman_service_lookup(name);
-
 	g_free(name);
 
 	return service;
@@ -1443,15 +1673,20 @@ unsigned int __connman_service_get_order(struct connman_service *service)
 	if (service == NULL)
 		return 0;
 
-	if (service->favorite == FALSE)
-		return 0;
+	if (service->favorite == FALSE) {
+		service->order = 0;
+		goto done;
+	}
 
 	iter = g_hash_table_lookup(service_hash, service->identifier);
 	if (iter != NULL) {
 		if (g_sequence_iter_get_position(iter) == 0)
-			return 1;
+			service->order = 1;
+		else
+			service->order = 0;
 	}
 
+done:
 	return service->order;
 }
 
@@ -1553,7 +1788,7 @@ static void update_from_network(struct connman_service *service,
 }
 
 /**
- * connman_service_create_from_network:
+ * __connman_service_create_from_network:
  * @network: network structure
  *
  * Look up service by network and if not found, create one
@@ -1564,12 +1799,6 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 	const char *ident, *group;
 	char *name;
 
-	service = __connman_service_lookup_from_network(network);
-	if (service != NULL) {
-		update_from_network(service, network);
-		return service;
-	}
-
 	ident = __connman_network_get_ident(network);
 	if (ident == NULL)
 		return NULL;
@@ -1578,24 +1807,21 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 	if (group == NULL)
 		return NULL;
 
-	if (__connman_network_get_weakness(network) == TRUE)
-		return NULL;
-
 	name = g_strdup_printf("%s_%s_%s",
 			__connman_network_get_type(network), ident, group);
-
 	service = __connman_service_get(name);
+	g_free(name);
+
 	if (service == NULL)
-		goto done;
+		return NULL;
+
+	if (__connman_network_get_weakness(network) == TRUE)
+		return service;
 
 	if (service->path != NULL) {
 		update_from_network(service, network);
-
 		__connman_profile_changed(TRUE);
-
-		__connman_service_put(service);
-		service = NULL;
-		goto done;
+		return service;
 	}
 
 	service->type = convert_network_type(network);
@@ -1606,15 +1832,46 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 
 	service_register(service);
 
+	__connman_profile_changed(TRUE);
+
 	if (service->favorite == TRUE)
 		__connman_service_auto_connect();
 
-done:
-	g_free(name);
-
 	return service;
 }
 
+void __connman_service_update_from_network(struct connman_network *network)
+{
+	struct connman_service *service;
+	connman_uint8_t strength;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return;
+
+	if (service->network == NULL)
+		return;
+
+	strength = connman_network_get_uint8(service->network, "Strength");
+	if (strength == service->strength)
+		return;
+
+	service->strength = strength;
+
+	strength_changed(service);
+}
+
+void __connman_service_remove_from_network(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return;
+
+	__connman_service_put(service);
+}
+
 static int service_load(struct connman_service *service)
 {
 	GKeyFile *keyfile;
@@ -1838,7 +2095,7 @@ update:
 		g_free(str);
 	}
 
-	if (service->passphrase != NULL)
+	if (service->passphrase != NULL && strlen(service->passphrase) > 0)
 		g_key_file_set_string(keyfile, service->identifier,
 					"Passphrase", service->passphrase);
 	else
